@@ -1,5 +1,5 @@
 /*
- * "$Id: gdevcups.c,v 1.10 2003/02/05 03:15:21 easysw Exp $"
+ * "$Id: gdevcups.c,v 1.15 2003/08/27 18:10:21 easysw Exp $"
  *
  *   GNU Ghostscript raster output driver for the Common UNIX Printing
  *   System (CUPS).
@@ -1935,6 +1935,8 @@ cups_put_params(gx_device     *pdev,	/* I - Device info */
   int			old_depth;	/* Old color depth */
   int			size_set;	/* Was the size set? */
   gdev_prn_space_params	sp;		/* Space parameter data */
+  int			width,		/* New width of page */
+			height;		/* New height of page */
 
 
 #ifdef DEBUG
@@ -2228,27 +2230,49 @@ cups_put_params(gx_device     *pdev,	/* I - Device info */
 
     if (cups->landscape)
     {
-      pdev->width  = (pdev->MediaSize[1] - pdev->HWMargins[0] - pdev->HWMargins[2]) *
-                     pdev->HWResolution[0] / 72.0f + 0.499f;
-      pdev->height = (pdev->MediaSize[0] - pdev->HWMargins[1] - pdev->HWMargins[3]) *
-                     pdev->HWResolution[1] / 72.0f + 0.499f;
+      width  = (pdev->MediaSize[1] - pdev->HWMargins[0] - pdev->HWMargins[2]) *
+               pdev->HWResolution[0] / 72.0f + 0.499f;
+      height = (pdev->MediaSize[0] - pdev->HWMargins[1] - pdev->HWMargins[3]) *
+               pdev->HWResolution[1] / 72.0f + 0.499f;
     }
     else
     {
-      pdev->width  = (pdev->MediaSize[0] - pdev->HWMargins[0] - pdev->HWMargins[2]) *
-                     pdev->HWResolution[0] / 72.0f + 0.499f;
-      pdev->height = (pdev->MediaSize[1] - pdev->HWMargins[1] - pdev->HWMargins[3]) *
-                     pdev->HWResolution[1] / 72.0f + 0.499f;
+      width  = (pdev->MediaSize[0] - pdev->HWMargins[0] - pdev->HWMargins[2]) *
+               pdev->HWResolution[0] / 72.0f + 0.499f;
+      height = (pdev->MediaSize[1] - pdev->HWMargins[1] - pdev->HWMargins[3]) *
+               pdev->HWResolution[1] / 72.0f + 0.499f;
     }
 
-    fprintf(stderr, "DEBUG: Reallocating memory, [%.0f %.0f] = %dx%d pixels...\n",
-            pdev->MediaSize[0], pdev->MediaSize[1], pdev->width, pdev->height);
+   /*
+    * Don't reallocate memory unless the device has been opened...
+    */
 
-    sp = ((gx_device_printer *)pdev)->space_params;
+    if (pdev->is_open)
+    {
+     /*
+      * Device is open, so reallocate...
+      */
 
-    if ((code = gdev_prn_reallocate_memory(pdev, &sp, pdev->width,
-                                           pdev->height)) < 0)
-      return (code);
+      fprintf(stderr, "DEBUG: Reallocating memory, [%.0f %.0f] = %dx%d pixels...\n",
+              pdev->MediaSize[0], pdev->MediaSize[1], width, height);
+
+      sp = ((gx_device_printer *)pdev)->space_params;
+
+      if ((code = gdev_prn_reallocate_memory(pdev, &sp, width, height)) < 0)
+	return (code);
+    }
+    else
+    {
+     /*
+      * Device isn't yet open, so just save the new width and height...
+      */
+
+      fprintf(stderr, "DEBUG: Setting initial media size, [%.0f %.0f] = %dx%d pixels...\n",
+              pdev->MediaSize[0], pdev->MediaSize[1], width, height);
+
+      pdev->width  = width;
+      pdev->height = height;
+    }
   }
 
 #ifdef DEBUG
@@ -2422,7 +2446,13 @@ cups_set_color_info(gx_device *pdev)	/* I - Device info */
   */
 
   for (i = 0; i <= gx_max_color_value; i ++)
-    lut_rgb_color[i] = cups->color_info.max_gray * i / gx_max_color_value;
+  {
+    lut_rgb_color[i] = (cups->color_info.max_gray * i + gx_max_color_value / 2) /
+                       gx_max_color_value;
+
+    if (i == 0 || lut_rgb_color[i] != lut_rgb_color[i - 1])
+      fprintf(stderr, "DEBUG2: lut_rgb_color[%d] = %d\n", i, lut_rgb_color[i]);
+  }
 
   for (i = 0; i < cups->color_info.dither_grays; i ++)
     lut_color_rgb[i] = gx_max_color_value * i / cups->color_info.max_gray;
@@ -3642,5 +3672,5 @@ cups_print_planar(gx_device_printer *pdev,	/* I - Printer device */
 
 
 /*
- * End of "$Id: gdevcups.c,v 1.10 2003/02/05 03:15:21 easysw Exp $".
+ * End of "$Id: gdevcups.c,v 1.15 2003/08/27 18:10:21 easysw Exp $".
  */
