@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: dwimg.c,v 1.3 2002/04/23 12:43:43 easysw Exp $ */
+/* $Id: dwimg.c,v 1.4 2003/07/13 04:43:17 easysw Exp $ */
 
 /* display device image window for Windows */
 
@@ -513,11 +513,15 @@ image_color(unsigned int format, int index,
 		    *r = *g = *b = (index ? 0 : 255);
 		    break;
 		case DISPLAY_DEPTH_4:
-		    {
-		    int one = index & 8 ? 255 : 128;
-		    *r = (index & 4 ? one : 0);
-		    *g = (index & 2 ? one : 0);
-		    *b = (index & 1 ? one : 0);
+		    if (index == 7)
+			*r = *g = *b = 170;
+		    else if (index == 8)
+			*r = *g = *b = 85;
+		    else {
+			int one = index & 8 ? 255 : 128;
+			*r = (index & 4 ? one : 0);
+			*g = (index & 2 ? one : 0);
+			*b = (index & 1 ? one : 0);
 		    }
 		    break;
 		case DISPLAY_DEPTH_8:
