@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: dxmain.c,v 1.3 2002/04/23 12:43:43 easysw Exp $ */
+/* $Id: dxmain.c,v 1.4 2003/07/13 04:43:17 easysw Exp $ */
 
 /* dxmain.c */
 /* 
@@ -65,6 +65,10 @@ enum SEPARATIONS {
     SEP_BLACK = 1
 };
 
+#ifndef min
+#define min(a,b) ((a) < (b) ? (a) : (b))
+#endif
+
 /*********************************************************************/
 /* stdio functions */
 
@@ -162,6 +166,7 @@ IMAGE *first_image;
 static IMAGE *image_find(void *handle, void *device);
 static void window_destroy(GtkWidget *w, gpointer data);
 static void window_create(IMAGE *img);
+static void window_resize(IMAGE *img);
 static gboolean window_draw(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
 
 static IMAGE *
@@ -184,48 +189,85 @@ window_draw(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
     if (img && img->window && img->buf) {
         int color = img->format & DISPLAY_COLORS_MASK;
 	int depth = img->format & DISPLAY_DEPTH_MASK;
-	switch (color) {
-	    case DISPLAY_COLORS_NATIVE:
-		if (depth == DISPLAY_DEPTH_8)
-		    gdk_draw_indexed_image(widget->window, 
-			widget->style->fg_gc[GTK_STATE_NORMAL],
-			0, 0, img->width, img->height,
-			GDK_RGB_DITHER_MAX, img->buf, img->rowstride,
-			img->cmap);
-	  	else if ((depth == DISPLAY_DEPTH_16) && img->rgbbuf)
-		    gdk_draw_rgb_image(widget->window, 
-			widget->style->fg_gc[GTK_STATE_NORMAL],
-			0, 0, img->width, img->height,
-			GDK_RGB_DITHER_MAX, img->rgbbuf, img->width * 3);
-		break;
-	    case DISPLAY_COLORS_GRAY:
-		if (depth == DISPLAY_DEPTH_8)
-		    gdk_draw_gray_image(widget->window, 
-			widget->style->fg_gc[GTK_STATE_NORMAL],
-			0, 0, img->width, img->height,
-			GDK_RGB_DITHER_MAX, img->buf, img->rowstride);
-		break;
-	    case DISPLAY_COLORS_RGB:
-		if (depth == DISPLAY_DEPTH_8) {
-		    if (img->rgbbuf)
+	int x, y, width, height;
+	if (event->area.x + event->area.width > img->width) {
+	    x = img->width;
+	    width = (event->area.x + event->area.width) - x;
+	    y = event->area.y;
+            height = min(img->height, event->area.y + event->area.height) - y;
+	    gdk_window_clear_area(widget->window, x, y, width, height);
+	}
+	if (event->area.y + event->area.height > img->height) {
+	    x = event->area.x;
+	    width = event->area.width;
+	    y = img->height;
+	    height = (event->area.y + event->area.height) - y;
+	    gdk_window_clear_area(widget->window, x, y, width, height);
+	}
+	x = event->area.x;
+	y = event->area.y;
+	width = event->area.width;
+	height = event->area.height;
+	if ((x>=0) && (y>=0) && (x < img->width) && (y < img->height)) {
+	    /* drawing area intersects bitmap */
+	    if (x + width > img->width)
+		width = img->width - x;
+	    if (y + height > img->height)
+		height =  img->height - y;
+	    switch (color) {
+		case DISPLAY_COLORS_NATIVE:
+		    if (depth == DISPLAY_DEPTH_8)
+			gdk_draw_indexed_image(widget->window, 
+			    widget->style->fg_gc[GTK_STATE_NORMAL],
+			    x, y, width, height, 
+			    GDK_RGB_DITHER_MAX, 
+			    img->buf + x + y*img->rowstride, 
+			    img->rowstride, img->cmap);
+		    else if ((depth == DISPLAY_DEPTH_16) && img->rgbbuf)
 			gdk_draw_rgb_image(widget->window, 
 			    widget->style->fg_gc[GTK_STATE_NORMAL],
-			    0, 0, img->width, img->height,
-			    GDK_RGB_DITHER_MAX, img->rgbbuf, img->width * 3);
-		    else
+			    x, y, width, height, 
+			    GDK_RGB_DITHER_MAX, 
+			    img->rgbbuf + x*3 + y*img->width*3, 
+			    img->width * 3);
+		    break;
+		case DISPLAY_COLORS_GRAY:
+		    if (depth == DISPLAY_DEPTH_8)
+			gdk_draw_gray_image(widget->window, 
+			    widget->style->fg_gc[GTK_STATE_NORMAL],
+			    x, y, width, height,
+			    GDK_RGB_DITHER_MAX, 
+			    img->buf + x + y*img->rowstride, 
+			    img->rowstride);
+		    break;
+		case DISPLAY_COLORS_RGB:
+		    if (depth == DISPLAY_DEPTH_8) {
+			if (img->rgbbuf)
+			    gdk_draw_rgb_image(widget->window, 
+				widget->style->fg_gc[GTK_STATE_NORMAL],
+				x, y, width, height,
+				GDK_RGB_DITHER_MAX, 
+				img->rgbbuf + x*3 + y*img->width*3, 
+				img->width * 3);
+			else
+			    gdk_draw_rgb_image(widget->window, 
+				widget->style->fg_gc[GTK_STATE_NORMAL],
+				x, y, width, height,
+				GDK_RGB_DITHER_MAX, 
+				img->buf + x*3 + y*img->rowstride, 
+				img->rowstride);
+		    }
+		    break;
+		case DISPLAY_COLORS_CMYK:
+		    if ((depth == DISPLAY_DEPTH_8) && img->rgbbuf)
 			gdk_draw_rgb_image(widget->window, 
 			    widget->style->fg_gc[GTK_STATE_NORMAL],
-			    0, 0, img->width, img->height,
-			    GDK_RGB_DITHER_MAX, img->buf, img->rowstride);
-		}
-		break;
-	    case DISPLAY_COLORS_CMYK:
-		if ((depth == DISPLAY_DEPTH_8) && img->rgbbuf)
-		    gdk_draw_rgb_image(widget->window, 
-			widget->style->fg_gc[GTK_STATE_NORMAL],
-			0, 0, img->width, img->height,
-			GDK_RGB_DITHER_MAX, img->rgbbuf, img->width * 3);
-		break;
+			    x, y, width, height,
+			    GDK_RGB_DITHER_MAX, 
+			    img->rgbbuf + x*3 + y*img->width*3, 
+			    img->width * 3);
+		    break;
+	    }
 	}
     }
     return TRUE;
@@ -244,7 +286,6 @@ static void window_create(IMAGE *img)
     /* Create a gtk window */
     img->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(img->window), "gs");
-    gtk_window_set_default_size(GTK_WINDOW(img->window), 500, 400);
     img->vbox = gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(img->window), img->vbox);
     gtk_widget_show(img->vbox);
@@ -265,6 +306,22 @@ static void window_create(IMAGE *img)
     /* do not show img->window until we know the image size */
 }
 
+static void window_resize(IMAGE *img)
+{
+    gtk_drawing_area_size(GTK_DRAWING_AREA (img->darea), 
+	img->width, img->height);
+    if (!(GTK_WIDGET_FLAGS(img->window) & GTK_VISIBLE)) {
+	/* We haven't yet shown the window, so set a default size 
+	 * which is smaller than the desktop to allow room for 
+	 * desktop toolbars, and if possible a little larger than 
+	 * the image to allow room for the scroll bars.
+	 * We don't know the width of the scroll bars, so just guess. */
+	gtk_window_set_default_size(GTK_WINDOW(img->window), 
+	    min(gdk_screen_width()-96, img->width+24),
+	    min(gdk_screen_height()-96, img->height+24));
+    }
+}
+
 static void window_separation(IMAGE *img, int layer)
 {
     img->separation ^= layer;
@@ -415,7 +472,6 @@ static int display_size(void *handle, void *device, int width, int height,
     img->rowstride = raster;
     img->buf = pimage;
     img->format = format;
-    gtk_drawing_area_size(GTK_DRAWING_AREA (img->darea), width, height);
 
     color = img->format & DISPLAY_COLORS_MASK;
     depth = img->format & DISPLAY_DEPTH_MASK;
@@ -504,7 +560,7 @@ static int display_size(void *handle, void *device, int width, int height,
 	    gtk_widget_hide(img->cmyk_bar);
     }
 
-
+    window_resize(img);
     if (!(GTK_WIDGET_FLAGS(img->window) & GTK_VISIBLE))
 	gtk_widget_show(img->window);
 
@@ -523,11 +579,6 @@ static int display_sync(void *handle, void *device)
     if (img == NULL)
 	return -1;
 
-    if (img->window == NULL)
-	window_create(img);
-    if (!(GTK_WIDGET_FLAGS(img->window) & GTK_VISIBLE))
-	gtk_widget_show_all(img->window);
-
     color = img->format & DISPLAY_COLORS_MASK;
     depth = img->format & DISPLAY_DEPTH_MASK;
     endian = img->format & DISPLAY_ENDIAN_MASK;
@@ -729,6 +780,13 @@ static int display_sync(void *handle, void *device)
 	    break;
     }
 
+    if (img->window == NULL) {
+	window_create(img);
+	window_resize(img);
+    }
+    if (!(GTK_WIDGET_FLAGS(img->window) & GTK_VISIBLE))
+	gtk_widget_show_all(img->window);
+
     gtk_widget_draw(img->darea, NULL);
     gtk_main_iteration_do(FALSE);
     return 0;
