@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevmac.c,v 1.4 2002/04/23 11:58:42 easysw Exp $ */
+/* $Id: gdevmac.c,v 1.5 2003/07/13 04:43:17 easysw Exp $ */
 
 #include "gdevmac.h"
 #include "gsparam.h"
@@ -149,7 +149,8 @@ mac_open(register gx_device *dev)
 	*mdev->currPicPos = 0x00ff;
 	
 	// notify the caller that a new device was opened
-	(*pgsdll_callback) (GSDLL_DEVICE, (char *)mdev, 1);
+	if (pgsdll_callback)
+		(*pgsdll_callback) (GSDLL_DEVICE, (char *)mdev, 1);
 	
 	return 0;
 }
@@ -179,7 +180,8 @@ mac_sync_output(gx_device * dev)
 	*mdev->currPicPos = 0x00ff;
 	
 	// tell the caller to sync
-	(*pgsdll_callback) (GSDLL_SYNC, (char *)mdev, 0);
+	if (pgsdll_callback)
+		(*pgsdll_callback) (GSDLL_SYNC, (char *)mdev, 0);
 	
 	return (0);
 }
@@ -200,7 +202,8 @@ mac_output_page(gx_device * dev, int copies, int flush)
 	}
 	
 	// tell the caller that the page is done
-	(*pgsdll_callback) (GSDLL_PAGE, (char *)mdev, 0);
+	if (pgsdll_callback)
+		(*pgsdll_callback) (GSDLL_PAGE, (char *)mdev, 0);
 	
 	gx_finish_output_page(dev, copies, flush);
 	
@@ -249,8 +252,9 @@ mac_close(register gx_device *dev)
 	}
 	
 	// notify the caller that the device was closed
-	// he has to dispose the PICT handle himself when he is ready!
-	(*pgsdll_callback) (GSDLL_DEVICE, (char *)mdev, 0);
+	// it has to dispose the PICT handle when it is ready!
+	if (pgsdll_callback)
+		(*pgsdll_callback) (GSDLL_DEVICE, (char *)mdev, 0);
 	
 	return 0;
 }
