@@ -1402,7 +1402,7 @@ mj_print_page(gx_device_printer * pdev, FILE * prn_stream, int ptype)
   return 0;
 }
 
-void 
+private void
 mj_color_correct(gx_color_value *Rptr ,gx_color_value *Gptr , gx_color_value *Bptr )
 								/* R,G,B : 0 to 255 */
 {
@@ -1745,6 +1745,7 @@ mjc_correct_color(gx_device_printer *pdev, gx_color_index ci)
     k3 = (k3 < cmask) ? k3 : cmask;
     k4 = (((ci >> 24) & cmask) * (mjb)) >> 10;
     k4 = (k4 < cmask) ? k4 : cmask;
+#if __WORDSIZE == 64
     /* This code is ugly... (for 64 bit OS) */
     if (sizeof(word) == 8)
       {
@@ -1762,6 +1763,7 @@ mjc_correct_color(gx_device_printer *pdev, gx_color_index ci)
 	  dprintf1("%s\n", "error");
 	return co;
       }
+#endif
     return k + (k2 << 8) + (k3 << 16) + (k << 24);
     break;
  case 32:
@@ -1773,7 +1775,7 @@ mjc_correct_color(gx_device_printer *pdev, gx_color_index ci)
     c = (c < cmask) ? c : cmask; 
     k = (((ci >> 24) & cmask) * mjb * dn) >> 20;
     k = (k < cmask) ? k : cmask; 
-
+#if __WORDSIZE == 64
     /* This code is ugly... (for 64 bit OS) */
     if (sizeof(word) == 8) 	
       {
@@ -1791,7 +1793,7 @@ mjc_correct_color(gx_device_printer *pdev, gx_color_index ci)
 
 	return co;
       }
-
+#endif
     co =    (y + (m << 8) + (c << 16) + (k << 24));
 /*     fprintf(stderr,"%d,%d:%d,%d,%d,%d\n", ci,co, y, m, c, k); */
     return co;
