@@ -16,7 +16,7 @@
 
   Author: Raph Levien <raph@artofcode.com>
 */
-/*$Id: gdevp14.c,v 1.3 2002/04/23 12:43:43 easysw Exp $ */
+/*$Id: gdevp14.c,v 1.4 2003/07/13 04:43:17 easysw Exp $ */
 /* Device filter implementing PDF 1.4 imaging model */
 
 #include "math_.h"
@@ -122,8 +122,8 @@ private dev_proc_text_begin(pdf14_text_begin);
 private dev_proc_begin_transparency_group(pdf14_begin_transparency_group);
 private dev_proc_end_transparency_group(pdf14_end_transparency_group);
 
-#define XSIZE (8.5 * X_DPI)	/* 8.5 x 11 inch page, by default */
-#define YSIZE (11 * Y_DPI)
+#define XSIZE (int)(8.5 * X_DPI)	/* 8.5 x 11 inch page, by default */
+#define YSIZE (int)(11 * Y_DPI)
 
 /* 24-bit color. */
 
@@ -361,19 +361,22 @@ pdf14_ctx_new(gs_int_rect *rect, int n_chan, gs_memory_t *memory)
 
     result = gs_alloc_struct(memory, pdf14_ctx, &st_pdf14_ctx,
 			     "pdf14_ctx_new");
+    if (result == NULL)
+	return result;
 
     buf = pdf14_buf_new(rect, false, false, n_chan, memory);
     if (buf == NULL) {
 	gs_free_object(memory, result, "pdf14_ctx_new");
 	return NULL;
     }
+    if_debug3('v', "[v]base buf: %d x %d, %d channels\n",
+	      buf->rect.q.x, buf->rect.q.y, buf->n_chan);
+    memset(buf->data, 0, buf->planestride * buf->n_planes);
     buf->saved = NULL;
     result->stack = buf;
     result->n_chan = n_chan;
     result->memory = memory;
     result->rect = *rect;
-    if (result == NULL)
-	return result;
     return result;
 }
 
@@ -430,6 +433,7 @@ pdf14_push_transparency_group(pdf14_ctx *ctx, gs_int_rect *rect,
     has_shape = tos->has_shape || tos->knockout;
 
     buf = pdf14_buf_new(rect, !isolated, has_shape, ctx->n_chan, ctx->memory);
+    if_debug3('v', "[v]push buf: %d x %d, %d channels\n", buf->rect.p.x, buf->rect.p.y, buf->n_chan);
     if (buf == NULL)
 	return_error(gs_error_VMerror);
     buf->isolated = isolated;
@@ -577,6 +581,7 @@ pdf14_pop_transparency_group(pdf14_ctx *ctx)
     }
 
     ctx->stack = nos;
+    if_debug0('v', "[v]pop buf\n");
     pdf14_buf_free(tos, ctx->memory);
     return 0;
 }
@@ -636,15 +641,15 @@ pdf14_put_image(pdf14_device *pdev, gs_state *pgs, gx_device *target)
     gx_set_dev_color(pgs);
     gs_image_t_init_adjust(&image, pis->shared->device_color_spaces.named.RGB,
 			   false);
-    image.ImageMatrix.xx = width;
-    image.ImageMatrix.yy = height;
+    image.ImageMatrix.xx = (float)width;
+    image.ImageMatrix.yy = (float)height;
     image.Width = width;
     image.Height = height;
     image.BitsPerComponent = 8;
-    pmat.xx = width;
+    pmat.xx = (float)width;
     pmat.xy = 0;
     pmat.yx = 0;
-    pmat.yy = height;
+    pmat.yy = (float)height;
     pmat.tx = 0;
     pmat.ty = 0;
     code = dev_proc(target, begin_typed_image) (target,
@@ -683,7 +688,7 @@ pdf14_put_image(pdf14_device *pdev, gs_state *pgs, gx_device *target)
 		g += (tmp + (tmp >> 8)) >> 8;
 		linebuf[x * 3 + 1] = g;
 
-		tmp = ((bg_g - b) * a) + 0x80;
+		tmp = ((bg_b - b) * a) + 0x80;
 		b += (tmp + (tmp >> 8)) >> 8;
 		linebuf[x * 3 + 2] = b;
 	    } else if (a == 0) {
@@ -1074,9 +1079,9 @@ pdf14_mark_fill_rectangle(gx_device * dev,
     src[0] = color >> 16;
     src[1] = (color >> 8) & 0xff;
     src[2] = color & 0xff;
-    src[3] = floor (255 * mdev->alpha + 0.5);
+    src[3] = (byte)floor (255 * mdev->alpha + 0.5);
     if (has_shape)
-	shape = floor (255 * mdev->shape + 0.5);
+	shape = (byte)floor (255 * mdev->shape + 0.5);
 
     if (x < buf->rect.p.x) x = buf->rect.p.x;
     if (y < buf->rect.p.x) y = buf->rect.p.y;
@@ -1128,8 +1133,8 @@ pdf14_mark_fill_rectangle_ko_simple(gx_device * dev,
     src[0] = color >> 16;
     src[1] = (color >> 8) & 0xff;
     src[2] = color & 0xff;
-    src[3] = floor (255 * mdev->shape + 0.5);
-    opacity = floor (255 * mdev->opacity + 0.5);
+    src[3] = (byte)floor (255 * mdev->shape + 0.5);
+    opacity = (byte)floor (255 * mdev->opacity + 0.5);
 
     if (x < buf->rect.p.x) x = buf->rect.p.x;
     if (y < buf->rect.p.x) y = buf->rect.p.y;
