@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevx.c,v 1.4 2002/04/23 11:58:43 easysw Exp $ */
+/*$Id: gdevx.c,v 1.5 2003/08/19 15:26:54 tillkamppeter Exp $ */
 /* X Windows driver for Ghostscript library */
 #include "gx.h"			/* for gx_bitmap; includes std.h */
 #include "math_.h"
@@ -228,6 +228,7 @@ x_device(gs_x11_device,
 				     24, 255, 256 ),
 	 0);
 
+#ifndef USE_BROKEN_X11ALPHA
 x_device(gs_x11alpha_device,
 	 std_device_dci_alpha_type_body(gx_device_X, 0, "x11alpha", &st_device_X,
 					FAKE_RES * DEFAULT_WIDTH_10THS / 10,
@@ -235,6 +236,7 @@ x_device(gs_x11alpha_device,
 					FAKE_RES, FAKE_RES,	/* x and y density (nominal) */
 					3, 24, 255, 255, 256, 256, 4, 4 ),
 	 50000000);
+#endif
 
 /* If XPutImage doesn't work, do it ourselves. */
 private int alt_put_image(P11(gx_device * dev, Display * dpy, Drawable win,
@@ -1035,24 +1037,34 @@ update_do_flush(gx_device_X * xdev)
     if (xdev->update.count != 0) {
 	int x = xdev->update.box.p.x, y = xdev->update.box.p.y;
 	int w = xdev->update.box.q.x - x, h = xdev->update.box.q.y - y;
+	const gx_device_memory *mdev = NULL;
+
+	/*
+	 * Copy from memory image to X server if any.
+	 */
+	if (xdev->is_buffered) {
+	    /*
+	     * The bbox device may have set the target to NULL
+	     * temporarily.  If this is the case, defer the screen
+	     * update.
+	     */
+	    if (!(mdev = (const gx_device_memory *)xdev->target))
+		return;
+	}
+
+	/*
+	 * mdev->width and mdev->height arn't the same as
+	 * xdev->width and xdev->height ... at least for gv
+	 */
+	if (mdev)
+	    fit_fill_xywh(mdev, x, y, w, h);
+	else
+	    fit_fill_xywh(xdev, x, y, w, h);
 
-	fit_fill_xywh(xdev, x, y, w, h);
 	if (w > 0 && h > 0) {
-	    if (xdev->is_buffered) {
-		/* Copy from memory image to X server. */
-		const gx_device_memory *mdev =
-		    (const gx_device_memory *)xdev->target;
-
-		/*
-		 * The bbox device may have set the target to NULL
-		 * temporarily.  If this is the case, defer the screen
-		 * update.
-		 */
-		if (mdev == NULL)
-		    return;	/* don't reset */
+	    if (mdev)
 		x_copy_image(xdev, mdev->line_ptrs[y], x, mdev->raster,
 			     x, y, w, h);
-	    }
 	    if (xdev->bpixmap) {
 		/* Copy from X backing pixmap to screen. */
 
