@@ -91,6 +91,7 @@
 /* Include these first.  Ghostscript redefines printf */
 #include <glib.h>
 #include <gmodule.h>
+#include <stdint.h>
 
 #include "gdevprna.h"
 #include "gdevpccm.h"
@@ -360,7 +361,7 @@ OpenDevice (gx_device *pdev  /* Driver instance to open */)
    int                    min_band_height;
    int                    max_src_image_row;
 
-   if (fDebugOutput) fprintf (stderr, "OpenDevice: pDev = %08X, pDev->hmodOmni = %08X\n", (int)pDev, (int)pDev->hmodOmni);
+   if (fDebugOutput) fprintf (stderr, "OpenDevice: pDev = %p, pDev->hmodOmni = %p\n", pDev, pDev->hmodOmni);
 
    if (!pDev)
    {
@@ -460,7 +461,7 @@ CloseDevice (gx_device * pdev)
    gx_device_omni * const pwdev = (gx_device_omni *)pdev;
    PDEVSTRUCT             pDev  = pwdev->pDev;
 
-   if (fDebugOutput) fprintf (stderr, "CloseDevice: pDev = %08X, pDev->pcoreOmni->pDevice = %08X\n", (int)pDev, (int)pDev->pcoreOmni->pDevice);
+   if (fDebugOutput) fprintf (stderr, "CloseDevice: pDev = %p, pDev->pcoreOmni->pDevice = %p\n", pDev, pDev->pcoreOmni->pDevice);
 
    if (!pDev)
    {
@@ -549,7 +550,7 @@ GetDeviceParams (gx_device *pgxdev, gs_param_list *plist)
    unsigned long           ulPelSizeCorrection = 100;
    bool                    bPersist            = false;
 
-   if (fDebugOutput) fprintf (stderr, "GetDeviceParams: pDev = %08X\n", (int)pDev);
+   if (fDebugOutput) fprintf (stderr, "GetDeviceParams: pDev = %p\n", pDev);
 
    iReturnCode = gdev_prn_get_params (pgxdev, plist);
 
@@ -652,6 +653,11 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
    static char             cOmnilib[]         = "libomni.so";
    static char             cDialogName[]      = "libomnijobdialog.so";
    static const char      *apszLibraryPaths[] = {
+#if __WORDSIZE == 64
+      "/usr/lib64/ghostscript/",
+#else
+      "/usr/lib/ghostscript/",
+#endif
       "/usr/lib/Omni/",
       "/opt/Omni/bin/",
       "/opt/Omni/lib/",
@@ -661,7 +667,7 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
    PRINTMODE               prtMode;           /* definition of printed output Bpp ..*/
    gs_param_string         fname;
 
-   if (fDebugOutput) fprintf (stderr, "SetupDevice: pDev = %08X\n", (int)pDev);
+   if (fDebugOutput) fprintf (stderr, "SetupDevice: pDev = %p\n", pDev);
 
    /* Do we have a handle? */
    if (pDev == NULL)
@@ -700,7 +706,7 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
       pDev->pcoreOmni->cbSize = sizeof (core_omni_device);
    }
 
-   if (fDebugOutput) fprintf (stderr, "SetupDevice: pDev->hmodOmni = %08X\n", (int)pDev->hmodOmni);
+   if (fDebugOutput) fprintf (stderr, "SetupDevice: pDev->hmodOmni = %p\n", pDev->hmodOmni);
 
    /* Is the handle initialized? */
    if (!pDev->hmodOmni)
@@ -742,7 +748,7 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
 
               pDev->hmodOmni = g_module_open (pszDeviceLib, (GModuleFlags)0);
 
-              if (fDebugOutput) fprintf (stderr, "SetupDevice: Trying to load %s = 0x%08X\n", pszDeviceLib, (int)pDev->hmodOmni);
+              if (fDebugOutput) fprintf (stderr, "SetupDevice: Trying to load %s = %p\n", pszDeviceLib, pDev->hmodOmni);
           }
           gs_free (pszDeviceLib, strlen (pszDeviceLib) + 1, 1, "Devicestring");
       }
@@ -804,9 +810,9 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
 
          if (fDebugOutput)
             fprintf (stderr,
-                     "SetupDevice: g_module_symbol (%s) = 0x%08x\n",
+                     "SetupDevice: g_module_symbol (%s) = %p\n",
                      aeplEntryPoints[i].pszEntryPoint,
-                     (int)*aeplEntryPoints[i].pfnEntryPoint);
+                     *aeplEntryPoints[i].pfnEntryPoint);
       }
 
       if (  !pDev->pfnCreateDevice
@@ -1119,7 +1125,7 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
          return_error (gs_error_Fatal);  /* fail if no device was created */
       }
 
-      if (fDebugOutput) fprintf (stderr, "Device = %08X\n", (unsigned int)pDev->pcoreOmni->pDevice);
+      if (fDebugOutput) fprintf (stderr, "Device = %p\n", pDev->pcoreOmni->pDevice);
       if (fDebugOutput) fprintf (stderr, "Omni version = %s\n", pDev->pcoreOmni->cOmniVersion);
       if (fDebugOutput) fprintf (stderr, "Device = \"%s\"\n", pDev->pcoreOmni->cDeviceName);
 
