@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxclist.c,v 1.3 2002/04/23 11:58:45 easysw Exp $ */
+/*$Id: gxclist.c,v 1.5 2002/06/28 15:54:46 easysw Exp $ */
 /* Command list document- and page-level code. */
 #include "memory_.h"
 #include "string_.h"
@@ -159,7 +159,15 @@ clist_tile_cache_size(const gx_device * target, uint data_size)
     if (bits_size < min_bits_size)
 	bits_size = min_bits_size;
 #undef min_bits_size
-    return bits_size;
+
+   /*
+    * Make sure that the bits_size value is a nice round number, both here
+    * and below (and everywhere, really), since Ghostscript uses a lot of
+    * FORTRAN-like buffers rather than using real data structures or
+    * pointers.
+    */
+
+    return (bits_size + align_cached_bits_mod) & -align_cached_bits_mod;
 }
 
 /*
@@ -314,6 +322,7 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     if (code < 0)
 	return code;
     cdev->page_tile_cache_size = bits_size;
+    bits_size = (bits_size + align_cached_bits_mod) & -align_cached_bits_mod;
     data += bits_size;
     size -= bits_size;
     code = clist_init_bands(dev, &bdev, size, band_width, band_height);
