@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: imainarg.c,v 1.6 2002/05/17 20:29:29 easysw Exp $ */
+/*$Id: imainarg.c,v 1.10 2003/08/19 15:24:41 tillkamppeter Exp $ */
 /* Command line parsing and dispatching */
 #include "ctype_.h"
 #include "memory_.h"
@@ -338,6 +338,21 @@ run_stdin:
 		if (code < 0)
 		    return code;
 		pal->expand_ats = false;
+
+		if (arg[0]) {
+		    char *sarg;
+
+		    if (arg[0] == '@' ||
+			(arg[0] == '-' && !isdigit(arg[1]))
+			)
+			break;
+		    sarg = arg_heap_copy(arg);
+		    if (sarg == NULL)
+			return e_Fatal;
+		    code = runarg(minst, "", sarg, ".runstring", 0);
+		    if (code < 0)
+			return code;
+		}
 		while ((arg = arg_next(pal, &code)) != 0) {
 		    char *sarg;
 
@@ -361,8 +376,8 @@ run_stdin:
 		    arg_push_string(pal, p);
 		}
 		pal->expand_ats = ats;
-		break;
 	    }
+	    break;
 	case 'E':		/* log errors */
 	    switch (*arg) {
 		case 0:
@@ -668,8 +683,13 @@ esc_strcat(char *dest, const char *src)
 private int
 argproc(gs_main_instance * minst, const char *arg)
 {
-    gs_main_init1(minst);		/* need i_ctx_p to proceed */
-    minst->i_ctx_p->filearg = (unsigned char*) arg;	/* allow reading this file if SAFER set */
+    int code = gs_main_init1(minst);		/* need i_ctx_p to proceed */
+    char *filearg;
+    
+    if (code < 0)
+    	return code;
+    filearg = arg_heap_copy(arg);
+    minst->i_ctx_p->filearg = filearg;	/* allow reading this file if SAFER set */
     if (minst->run_buffer_size) {
 	/* Run file with run_string. */
 	return run_buffered(minst, arg);
@@ -828,8 +848,9 @@ private const char help_usage2[] = "\
                                          embed %d or %ld for page #\n";
 private const char help_trailer[] = "\
 For more information, see %s%sUse.htm.\n\
-Report bugs to %s, using the form in Bug-form.htm.\n";
+Report bugs using the form at \"http://www.cups.org/str.php\".\n";
 private const char help_devices[] = "Available devices:";
+private const char help_default_device[] = "Default output device:";
 private const char help_emulators[] = "Input formats:";
 private const char help_paths[] = "Search path:";
 
@@ -875,24 +896,55 @@ print_usage(const gs_main_instance *minst)
     outprintf("%s", help_usage2);
 }
 
+/* compare function for device name qsort */
+private int
+cmpstr(const void *v1, const void *v2)
+{
+    return strcmp( *(char * const *)v1, *(char * const *)v2 );
+}
+
 /* Print the list of available devices. */
 private void
 print_devices(const gs_main_instance *minst)
 {
+    outprintf("%s", help_default_device);
+    outprintf(" %s\n", gs_devicename(gs_getdevice(0)));
     outprintf("%s", help_devices);
     {
 	int i;
 	int pos = 100;
 	const gx_device *pdev;
+	const char **names;
+	size_t ndev = 0;
 
-	for (i = 0; (pdev = gs_getdevice(i)) != 0; i++) {
-	    const char *dname = gs_devicename(pdev);
-	    int len = strlen(dname);
+	for (i = 0; (pdev = gs_getdevice(i)) != 0; i++)
+	    ;
+	ndev = (size_t)i;
+	names = (const char **)gs_alloc_bytes(minst->heap, ndev * sizeof(const char*), "print_devices");
+	if (names == (const char **)NULL) { /* old-style unsorted device list */
+	    for (i = 0; (pdev = gs_getdevice(i)) != 0; i++) {
+		const char *dname = gs_devicename(pdev);
+		int len = strlen(dname);
 
-	    if (pos + 1 + len > 76)
-		outprintf("\n  "), pos = 2;
-	    outprintf(" %s", dname);
-	    pos += 1 + len;
+		if (pos + 1 + len > 76)
+		    outprintf("\n  "), pos = 2;
+		outprintf(" %s", dname);
+		pos += 1 + len;
+	    }
+	}
+	else {                          /* new-style sorted device list */
+	    for (i = 0; (pdev = gs_getdevice(i)) != 0; i++)
+		names[i] = gs_devicename(pdev);
+	    qsort((void*)names, ndev, sizeof(const char*), cmpstr);
+	    for (i = 0; i < ndev; i++) {
+		int len = strlen(names[i]);
+
+		if (pos + 1 + len > 76)
+		    outprintf("\n  "), pos = 2;
+		outprintf(" %s", names[i]);
+		pos += 1 + len;
+	    }
+	    gs_free((char *)names, ndev * sizeof(const char*), 1, "print_devices");
 	}
     }
     outprintf("\n");
