@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: iscan.c,v 1.3 2002/04/23 11:58:47 easysw Exp $ */
+/*$Id: iscan.c,v 1.4 2003/07/13 04:43:18 easysw Exp $ */
 /* Token scanner for Ghostscript interpreter */
 #include "ghost.h"
 #include "memory_.h"
@@ -308,7 +308,7 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
 #ifdef DEBUG
 	if (gs_debug_c('%')) {
 	    dlprintf2("[%%%%%s%c]", sstr, (len >= 3 ? '+' : '-'));
-	    fwrite(base, 1, len, dstderr);
+	    debug_print_string(base, len);
 	    dputs("\n");
 	}
 #endif
@@ -326,7 +326,7 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
     else {
 	if (gs_debug_c('%')) {
 	    dlprintf2("[%% %s%c]", sstr, (len >= 2 ? '+' : '-'));
-	    fwrite(base, 1, len, dstderr);
+	    debug_print_string(base, len);
 	    dputs("\n");
 	}
     }
@@ -365,6 +365,7 @@ scan_string_token_options(i_ctx_t *i_ctx_p, ref * pstr, ref * pref,
 
     if (!r_has_attr(pstr, a_read))
 	return_error(e_invalidaccess);
+    s_init(s, NULL);
     sread_string(s, pstr->value.bytes, r_size(pstr));
     scanner_state_init_options(&state, options | SCAN_FROM_STRING);
     switch (code = scan_token(i_ctx_p, s, pref, &state)) {
@@ -742,16 +743,14 @@ scan_token(i_ctx_t *i_ctx_p, stream * s, ref * pref, scanner_state * pstate)
 #define comment_line da.buf
 		--sptr;
 		comment_line[1] = 0;
-		if (scan_comment_proc != NULL ||
-		    ((sptr == base || base[1] == '%') &&
-		     scan_dsc_proc != NULL)
-		    ) {		/* Could be an externally processable comment. */
+		{
+		    /* Could be an externally processable comment. */
 		    uint len = sptr + 1 - base;
+		    if (len > sizeof(comment_line))
+			len = sizeof(comment_line);
 
 		    memcpy(comment_line, base, len);
 		    daptr = comment_line + len;
-		} else {	/* Not a DSC comment. */
-		    daptr = comment_line + (max_comment_line + 1);
 		}
 		da.base = comment_line;
 		da.is_dynamic = false;
@@ -785,7 +784,7 @@ scan_token(i_ctx_t *i_ctx_p, stream * s, ref * pref, scanner_state * pstate)
 		    case char_EOL:
 		    case '\f':
 		      end_comment:
-			retcode = scan_comment(i_ctx_p, pref, &sstate,
+			retcode = scan_comment(i_ctx_p, myref, &sstate,
 					       comment_line, daptr, true);
 			if (retcode != 0)
 			    goto comment;
@@ -1119,9 +1118,12 @@ scan_token(i_ctx_t *i_ctx_p, stream * s, ref * pref, scanner_state * pstate)
     }
   sret:if (retcode < 0) {
 	scan_end_inline();
-	if (pstack != 0)
+	if (pstack != 0) {
+	    if (retcode == e_undefined)
+		*pref = *osp;	/* return undefined name as error token */
 	    ref_stack_pop(&o_stack,
 			  ref_stack_count(&o_stack) - (pdepth - 1));
+	}
 	return retcode;
     }
     /* If we are at the top level, return the object, */
