@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.  
 */
 
-/* $Id: macsystypes.h,v 1.3 2002/04/23 11:58:47 easysw Exp $ */
+/* $Id: macsystypes.h,v 1.4 2003/07/13 04:43:18 easysw Exp $ */
 
 #ifndef __sys_types_h__
 #define __sys_types_h__
@@ -22,8 +22,6 @@
 #include <MacTypes.h>
 #include <unix.h>
 #define CHECK_INTERRUPTS
-#define SYSTIME_H
-#define HAVE_SYS_TIME_H
 
 #define main gs_main
 
