@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: pipe_.h,v 1.3 2002/04/23 11:58:47 easysw Exp $ */
+/*$Id: pipe_.h,v 1.4 2003/07/13 04:43:18 easysw Exp $ */
 /* Declaration of popen and pclose */
 
 #ifndef pipe__INCLUDED
@@ -25,8 +25,12 @@
 #ifdef __WIN32__
 /*
  * MS Windows has popen and pclose in stdio.h, but under different names.
+ * Unfortunately MSVC5 and 6 have a broken implementation of _popen, 
+ * so we use own.  Our implementation only supports mode "wb".
  */
-#  define popen(cmd, mode) _popen(cmd, mode)
+extern FILE *mswin_popen(const char *cmd, const char *mode);
+#  define popen(cmd, mode) mswin_popen(cmd, mode)
+/* #  define popen(cmd, mode) _popen(cmd, mode) */
 #  define pclose(file) _pclose(file)
 #else  /* !__WIN32__ */
 /*
