@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: zbfont.c,v 1.3 2002/04/23 11:58:48 easysw Exp $ */
+/*$Id: zbfont.c,v 1.4 2003/07/13 04:43:18 easysw Exp $ */
 /* Font creation utilities */
 #include "memory_.h"
 #include "string_.h"
@@ -345,10 +345,6 @@ build_gs_simple_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_base ** ppfont,
     code = font_bbox_param(op, bbox);
     if (code < 0)
 	return code;
-    if ((options & bf_FontBBox_required) &&
-	bbox[0] == 0 && bbox[1] == 0 && bbox[2] == 0 && bbox[3] == 0
-	)
-	return_error(e_invalidfont);
     code = dict_uid_param(op, &uid, 0, imemory, i_ctx_p);
     if (code < 0)
 	return code;
